require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xwefsubscription',
  dscmeta_resource_friendly_name: 'xWEFSubscription',
  dscmeta_resource_name: 'MSFT_xWEFSubscription',
  dscmeta_module_name: 'xWindowsEventForwarding',
  dscmeta_module_version: '1.0.0.0',
  docs: 'The DSC xWEFSubscription resource type.
         Automatically generated from version 1.0.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_locale: {
      type: 'Optional[String]',
      desc: 'Sets the subscription Locale, default en-US',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credentialstype: {
      type: "Optional[Enum['Default', 'Basic', 'Negotiate', 'Digest']]",
      desc: 'Sets the credential type used for authenticating to WinRM, default Default',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_description: {
      type: 'Optional[String]',
      desc: 'Description of the Collector subscription',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_contentformat: {
      type: 'Optional[String]',
      desc: 'Format that event logs will be submitted in, default RenderedText',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Determines whether to validate or remove the scubscription',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_maxitems: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'The number of events that can occur on the source before they are submitted to the collector, default 1',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_transportport: {
      type: 'Optional[String]',
      desc: 'Set the port number that WinRM should use to make a connection, default 5985',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_query: {
      type: 'Optional[Array[String]]',
      desc: 'Expects an array of hashtables that set which events should be collected, default is all application and system logs',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_address: {
      type: 'Optional[Array[String]]',
      desc: 'Expects an array of source node FQDNs, default source.wef.test to prevent errors when only staging test subscription',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_allowedsourcedomaincomputers: {
      type: 'Optional[String]',
      desc: 'In Source Initiated scenario this SDDL determines who can push events, default O:NSG:NSD:(A;;GA;;;DC)(A;;GA;;;NS) which equates to Domain Computers and Network Service',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_subscriptiontype: {
      type: "Enum['CollectorInitiated', 'SourceInitiated']",
      desc: 'Type of Subscription to create',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_maxlatencytime: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'The maximum amount of time that can pass before events are submitted to the collector, default 20000',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_subscriptionid: {
      type: 'String',
      desc: 'Name of the Subscription',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_deliverymode: {
      type: "Optional[Enum['Push', 'Pull']]",
      desc: 'Configures whether the collector will pull events from source nodes or if the source nodes will push events to the collector, default push',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_heartbeatinterval: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'Frequency to verify connectivity, default 20000',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_transportname: {
      type: "Optional[Enum['HTTP', 'HTTPS']]",
      desc: 'Determines whether to require SSL, default HTTP',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_readexistingevents: {
      type: "Optional[Enum['true', 'false']]",
      desc: 'Should the collector read existing or only new events, default false',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allowedsourcenondomaincomputers: {
      type: 'Optional[Array[String]]',
      desc: 'This parameter has not been fully implemented, only required for source initiated scenarios, provide XML to set IssuerCAList, AllowedSubjectList, or DeniedSubjectList if this will be used, default empty string',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_enabled: {
      type: "Optional[Enum['true', 'false']]",
      desc: 'Sets whether the subscription will be enabled, default true',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logfile: {
      type: 'Optional[String]',
      desc: 'Sets the event log that the collected events will be written to, default ForwardedEvents',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
